# GroupLineUpA. http://www.est.hi-ho.ne.jp/atamadekkati/ 
# グループ化,コンポーネント化された材を一列に並べる。
# 2009/07/19 ver1.00 作成
# 2009/08/02 ver1.10 寸法は回転前に削除することとする。
# 2009/08/16 ver1.50 イレクタージョイントのコンポーネント(ir#)は、特殊処理する。
# 2009/09/18 ver1.60 記載する名称をグループ名かレイヤ名か選択できるようにする。
#
# 使用者の自己責任でご使用下さい。
# 改変してマシなのができたら私にも教えてください。
#

require 'sketchup.rb'

#============= メインプログラム =============
def ozlineup
  model = Sketchup.active_model
  entities = model.entities
  bbox = model.bounds

# 初期設定
  @yadd = 20.mm                    #部材間の距離の初期値
  textarrowlen = 1                 #テキストを描画するときの引き出し線の長さを @yadd の何倍にするか。
  textarrowvec = [ -1 , 0 , 0 ]    #テキストを描画する位置の基準ベクトル(グループ名)
  textarrowvecsize = [ 1 , 0 , 0 ]    #テキストを描画する位置の基準ベクトル(グループサイズ)

# プロンプト
  prompts=["部材間の距離", "テキスト表示","部材の回転"]
  values=[@yadd, "レイヤ名", "回転"]
  enums=[nil, "グループ名|レイヤ名|非表示", "回転|そのまま"]
  results=inputbox prompts, values, enums, "グループごとにバラバラにします。事前に保存を。"
  return if not results
  @yadd=results[0]
  textarrowlen = @yadd * textarrowlen
  @textarrowvector = [textarrowvec[0] * textarrowlen , textarrowvec[1] * textarrowlen , textarrowvec[2] * textarrowlen ]
  @textarrowvectorsize = [textarrowvecsize[0] * textarrowlen , textarrowvecsize[1] * textarrowlen , textarrowvecsize[2] * textarrowlen ]
  if results[1] == "グループ名" then
    textnamesw=1 
  elsif results[1] == "レイヤ名" then
    textnamesw=2 
  else
  	textnamesw=0
  end
  if results[2] == "回転" then
    rotatesw=1 
  elsif results[2] == "そのまま" then
  	rotatesw=0
  else    # 使わないことにする。
    rotatesw=2
    mboxresult = UI.messagebox "グループ作成時のサイズと向きに戻します。よろしいですか。", MB_OKCANCEL
    return if mboxresult == 2
  end

  Sketchup::set_status_text("group 1 line, please wait...", SB_PROMPT) 
  Sketchup.active_model.start_operation("部材並べ")

# モデル内の数をカウント、削除対象（寸法）をチェック
  nr_edges = nr_faces = nr_componentinstances = nr_groups = nr_images = nr_sectionplanes = nr_dimensions = nr_texts = nr_clines = nr_cpoints = 0
  deleteents = []
  entities.each {|ent|
    type = ent.typename
    case type
      when "Edge"
        nr_edges = nr_edges + 1
      when "Face"
        nr_faces = nr_faces + 1
      when "ComponentInstance"
        nr_componentinstances = nr_componentinstances + 1
      when "Group"
        nr_groups = nr_groups + 1
      when "Image"
        nr_images = nr_images + 1
      when "SectionPlane"
        nr_sectionplanes = nr_sectionplanes + 1
      when "DimensionLinear" 
        nr_dimensions = nr_dimensions + 1
        deleteents.push ent
      when "DimensionRadial" 
        nr_dimensions = nr_dimensions + 1
        deleteents.push ent
      when "Text"
        nr_texts = nr_texts + 1
      when "ConstructionLine"
        nr_clines = nr_clines + 1
      when "ConstructionPoint"
        nr_cpoints = nr_cpoints + 1
    end
  }

# 削除対象（寸法）を削除
	entities.erase_entities deleteents

# グループとコンポーネントを回転
  irectorents = []
	yzahyo = nilname = 0
  entities.each {|e|
    type = e.typename
    case type
      when "ComponentInstance"
        if e.definition.name[0,3] == "ir#" then
          irectorents.push e
        else
          if rotatesw != 0 then
#                if rotatesw == 2 then
            rotateorigin( e )
#                end
            rotate90( e )
          end
          transpt = Geom::Point3d.new 0, yzahyo, 0
          enttrans( e , transpt )
          if textnamesw != 0 then
            textwrite( e , entities , textnamesw )
          end
          yzahyo = e.bounds.max.y + @yadd
        end
      when "Group"
        if rotatesw != 0 then
#          if rotatesw == 2 then
          rotateorigin( e )
#          end
          rotate90( e )
        end
        transpt = Geom::Point3d.new 0, yzahyo, 0
        enttrans( e , transpt )
        if textnamesw != 0 then
          textwrite( e , entities , textnamesw )
        end
        yzahyo = e.bounds.max.y + @yadd
    end
  }

# イレクター(ir#)のコンポーネントは回転せずに同種コンポーネント単位で並べる
# 　ただし、回転しないと次のコンポーネントと重なる恐れがある。
  ircompns = 0
  ircompname = []
  ircompyzahyo = [yzahyo]
  ircompxzahyo = [0]
  irectorents.each {|irent|
    irns = ircompname.index(irent.definition.name)
    if irns == nil then
      ircompname.push irent.definition.name  # 配列０から名前を入れていく
      yzahyo = ircompyzahyo[ircompns]
      xzahyo = 0
      ircompxzahyo[ircompns] = 0 + irent.bounds.max.x - irent.bounds.min.x + @yadd
      ircompns = ircompns + 1
      ircompyzahyo.push yzahyo + irent.bounds.max.y - irent.bounds.min.y + @yadd
      ircompxzahyo.push 0
    else
      yzahyo = ircompyzahyo[irns]
      xzahyo = ircompxzahyo[irns]
      ircompxzahyo[irns] = xzahyo + irent.bounds.max.x - irent.bounds.min.x + @yadd
# UI.messagebox(irent.definition.name + " " + xzahyo.to_s + " "  + " " + yzahyo.to_s )
    end
    transpt = Geom::Point3d.new xzahyo, yzahyo, 0
    enttrans( irent , transpt )
    if textnamesw == 1 then
      if irns == nil then
        textpoint = Geom::Point3d.new irent.bounds.min.x , ( irent.bounds.max.y + irent.bounds.min.y ) / 2 , ( irent.bounds.max.z + irent.bounds.min.z ) / 2
        enttext = entities.add_text irent.definition.name , textpoint , @textarrowvector
      end
    end
  }

#  視点の変更
  Sketchup.send_action("viewTop:")
  Sketchup.send_action("viewZoomExtents:")

#  メッセージ
  Sketchup::set_status_text("group 1 line, please wait......Done.", SB_PROMPT) 
  UI.messagebox("終了しました。" +
                "線(" + nr_edges.to_s + ")や面(" + nr_faces.to_s + ")などは移動しません。" +
                "寸法(" + nr_dimensions.to_s + ")は削除されます。" ) 

  Sketchup.active_model.commit_operation

end #of def

#============= 定義時からサイズ変更なければ、定義時の角度に回転する =============
def rotateorigin( e )

  transorigin = e.transformation.clone
#  サイズ変更していないかx,y,zの順にチェックする。
  vectorkijyun = Geom::Vector3d.new 1,0,0
  transvector = vectorkijyun.transform! transorigin
  sabun = transvector.length - 1
  if sabun.abs < 0.000001 then
    vectorkijyun = Geom::Vector3d.new 0,1,0
    transvector = vectorkijyun.transform! transorigin
    sabun = transvector.length - 1
    if sabun.abs < 0.000001 then
      vectorkijyun = Geom::Vector3d.new 0,0,1
      transvector = vectorkijyun.transform! transorigin
      sabun = transvector.length - 1
      if sabun.abs < 0.000001 then
        e.transform!(transorigin.inverse)
      end
    end
  end

end

#============= 箱の各辺の長さがＸ、Ｙ、Ｚの順になるよう90度回転する =============
def rotate90( e )
  eboundx = e.bounds.max.x - e.bounds.min.x
  eboundy = e.bounds.max.y - e.bounds.min.y
  eboundz = e.bounds.max.z - e.bounds.min.z
#        
  if eboundz > eboundy and eboundz > eboundx then
    trti = Geom::Transformation.rotation(e.bounds.min, [0, 1, 0] , 90.degrees)
  	e.transform!(trti)
#   	e.name = e.name + ",Y+90"
   	if eboundx > eboundy then
      trti = Geom::Transformation.rotation(e.bounds.min, [1, 0, 0] , 90.degrees)
     	e.transform!(trti)
#     	e.name = e.name + ",X+90"
    end
  elsif eboundy > eboundx and eboundy > eboundz then
    trti = Geom::Transformation.rotation(e.bounds.min, [0, 0, 1] , 90.degrees)
    e.transform!(trti)
#   	e.name = e.name + ",Z+90"
 	  if eboundz > eboundx then
	    trti = Geom::Transformation.rotation(e.bounds.min, [1, 0, 0] , 90.degrees)
      e.transform!(trti)
#     	e.name = e.name + ",X+90"
    end
  elsif eboundz > eboundy then
    trti = Geom::Transformation.rotation(e.bounds.min, [1, 0, 0] , 90.degrees)
    e.transform!(trti)
#   	e.name = e.name + ",X+90"
  end

end

#============= 箱の左下座標を指定した座標に平行移動する =============
def enttrans( e , transpt )

  trvec = Geom::Vector3d.new(transpt-e.bounds.min)
  trts = Geom::Transformation.translation(trvec)
  e.transform!(trts)

end

#============= 箱の名前と箱のサイズをテキストで描く =============
def textwrite( e , entities , textnamesw )

 # グループ名かレイヤ名
  textpoint = Geom::Point3d.new e.bounds.min.x , ( e.bounds.max.y + e.bounds.min.y ) / 2 , ( e.bounds.max.z + e.bounds.min.z ) / 2
  if textnamesw == 1 then
	  enttext = entities.add_text e.name , textpoint , @textarrowvector
	else
	  enttext = entities.add_text e.layer.name , textpoint , @textarrowvector
	end
  enttext.leader_type = 1

 # グループサイズ
  textpoint = Geom::Point3d.new e.bounds.max.x , ( e.bounds.max.y + e.bounds.min.y ) / 2 , ( e.bounds.max.z + e.bounds.min.z ) / 2
  enttext = entities.add_text e.bounds.width.to_l.to_s + "×" + e.bounds.height.to_l.to_s + "×" +e.bounds.depth.to_l.to_s , textpoint , @textarrowvectorsize
  enttext.leader_type = 1

end

#=============================================================================
# Add a menu 
if( not $ozlineup_menu_loaded )
  add_separator_to_menu("Tools")
  ozlineup_menu = UI.menu("Tools").add_item("部材並べ") { ozlineup }
  $ozlineup_menu_loaded = true
end
